'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * 流程节点关系数组生成
 * @param node 当前选中的流程节点
 * @returns {*}
 */
var createEdges = function createEdges(node) {
    switch (node.isForward) {
        case '0':
            // 0表示节点是当前节点，当前节点找上下游
            var sourceEdges = node.source.map(function (item) {
                return { source: item, target: node.id };
            });
            var targetEdges = node.target.map(function (item) {
                return { source: node.id, target: item };
            });
            return [].concat(_toConsumableArray(sourceEdges), _toConsumableArray(targetEdges));
        case '1':
            // 1表示下游节点，下游节点找下游
            return node.target.map(function (item) {
                return { source: node.id, target: item };
            });
        case '-1':
            // -1表示上游节点，上游节点找上游
            return node.source.map(function (item) {
                return { source: item, target: node.id };
            });
        default:
            return [];
    }
};

/**
 * 查找上下游的节点和路径
 * @param nodeId
 * @param data
 * @returns {{nodes: *[], edges: Array}}
 */
var getRelated = function getRelated(nodeId, data) {
    var edges = [];
    var sources = [];
    var targets = [];
    var getSource = function getSource(id) {
        var _edges;

        var sourceEdges = data.edges.filter(function (_ref) {
            var target = _ref.target;
            return target === id;
        });
        var sourceIds = sourceEdges.map(function (item) {
            return item.source;
        });
        (_edges = edges).push.apply(_edges, _toConsumableArray(sourceEdges));
        sources.push.apply(sources, _toConsumableArray(sourceIds));
        if (sourceIds.length > 0) {
            sourceIds.forEach(function (item) {
                return getSource(item);
            });
        }
    };
    var getTargets = function getTargets(id) {
        var _edges2;

        var targetEdges = data.edges.filter(function (_ref2) {
            var source = _ref2.source;
            return source === id;
        });
        var targetIds = targetEdges.map(function (item) {
            return item.target;
        });
        (_edges2 = edges).push.apply(_edges2, _toConsumableArray(targetEdges));
        targets.push.apply(targets, _toConsumableArray(targetIds));
        if (targetIds.length > 0) {
            targetIds.forEach(function (item) {
                return getTargets(item);
            });
        }
    };
    getSource(nodeId);
    getTargets(nodeId);
    edges = [].concat(_toConsumableArray(new Set(edges)));
    var nodeIds = [].concat(_toConsumableArray(new Set([].concat(sources, [nodeId], targets)))); // 用 Set 去重
    var nodes = nodeIds.map(function (item) {
        return data.nodes.find(function (ele) {
            return ele.id === item;
        });
    });
    return {
        edges: edges,
        nodes: nodes
    };
};

/**
 * 根据当前选中的流程节点ID查找上下游单据
 * @param selectId
 * @param data
 * @returns {Array}
 */
var getSourceTargetNode = function getSourceTargetNode(selectId, data) {
    var currentItem = data.find(function (item) {
        return item.lightBillVO.billID === selectId;
    });
    var sourceIds = [],
        //上游数据id
    sourceNodes = [],
        //上游数据lightBillVO
    targetIds = [],
        //下游数据id
    targetNodes = []; //下游数据lightBillVO
    if (currentItem && currentItem.isForward === '0') {
        //有上下游
        sourceIds = currentItem.sourceList;
        if (sourceIds && sourceIds instanceof Array) {
            sourceNodes = sourceIds.map(function (id) {
                return data.find(function (item) {
                    return item.lightBillVO.billID === id;
                }).lightBillVO;
            });
        }
        targetIds = currentItem.forwardList;
        if (targetIds && targetIds instanceof Array) {
            targetNodes = targetIds.map(function (id) {
                return data.find(function (item) {
                    return item.lightBillVO.billID === id;
                }).lightBillVO;
            });
        }
    }
    if (currentItem && currentItem.isForward === '-1') {
        //只有上游没有下游
        sourceIds = currentItem.sourceList;
        if (sourceIds && sourceIds instanceof Array) {
            sourceNodes = sourceIds.map(function (id) {
                return data.find(function (item) {
                    return item.lightBillVO.billID === id;
                }).lightBillVO;
            });
        }
        data.forEach(function (item) {
            if (item.sourceList && item.sourceList.includes(selectId)) {
                targetNodes.push(item.lightBillVO);
            }
        });
    }
    if (currentItem && currentItem.isForward === '1') {
        //只有下游没有上游
        targetIds = currentItem.forwardList;
        if (targetIds && targetIds instanceof Array) {
            targetNodes = targetIds.map(function (id) {
                return data.find(function (item) {
                    return item.lightBillVO.billID === id;
                }).lightBillVO;
            });
        }
        data.forEach(function (item) {
            if (item.forwardList && item.forwardList.includes(selectId)) {
                sourceNodes.push(item.lightBillVO);
            }
        });
    }
    var rows = [];
    var index = 1;
    sourceNodes.forEach(function (item) {
        item.isForward = '上游';
        item.index = index++;
        rows.push(item);
    });
    targetNodes.forEach(function (item) {
        item.isForward = '下游';
        item.index = index++;
        rows.push(item);
    });
    return rows;
};

/**
 * 处理单据类型名称和单据编号超长的场景
 * @param billTypeName
 * @param billCode
 * @returns {string}
 */
var suitableLabelText = function suitableLabelText(billTypeName, billCode) {
    var billcodeLen = textSize(billCode).width,
        halfCode1 = void 0,
        halfCode2 = void 0,
        newHalfCode2 = void 0,
        billCodeText = void 0,
        billTypeNameText = void 0;
    if (billcodeLen > 130) {
        //单据编号超过一行
        halfCode1 = billCode.slice(0, 16);
        halfCode2 = billCode.slice(16, billCode.length);
        var halfCode2Len = textSize(halfCode2).width;
        if (halfCode2Len > 130) {
            //据编号超过两行
            newHalfCode2 = halfCode2.slice(0, 15) + '...';
        }
    } else {
        //单据编号没有超过一行
        halfCode1 = billCode;
    }
    billCodeText = halfCode1 + '\n' + (newHalfCode2 || halfCode2 || '');

    if (billTypeName.length > 10) {
        //单据标题极值10个字符，超出..
        billTypeNameText = billTypeName.slice(0, 10) + '...';
    }
    return (billTypeNameText || billTypeName) + '\n' + billCodeText;
};

/**
 * 计算字符串的长度
 * fontSize:字体大小  String
 * fontFamily：字体系列  String
 * text：字符串  String
 */
var textSize = function textSize(text) {
    var fontSize = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '12px';
    var fontFamily = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '-apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Helvetica, "PingFang SC", "Hiragino Sans GB", "Microsoft YaHei"';

    var span = document.createElement('span');
    var result = {};
    result.width = span.offsetWidth;
    result.height = span.offsetHeight;
    span.style.visibility = 'hidden';
    span.style.fontSize = fontSize;
    span.style.fontFamily = fontFamily;
    span.style.display = 'inline-block';
    document.body.appendChild(span);
    if (typeof span.textContent != 'undefined') {
        span.textContent = text;
    } else {
        span.innerText = text;
    }
    result.width = parseFloat(window.getComputedStyle(span).width) - result.width;
    result.height = parseFloat(window.getComputedStyle(span).height) - result.height;
    return result;
};

// 测试数据
var dagreData = [{
    "lightBillVO": {
        "transtype": "Z2-02",
        "transtypepk": "0001A81000000000365P",
        "transTypeName": "项目采购合同",
        "billCode": "Z22020072800000001",
        "billID": "1001A810000000006JFM",
        "billType": "Z2",
        "billTypeName": "采购合同",
        "billCorp": "0001A810000000002QUK",
        "isDirect": true,
        "key": "Z22020072800000001Z21001A810000000006JFM",
        "sourceMap": {},
        "forwardMap": {
            "null211001A810000000006JGS": {
                "transtype": "21-01",
                "transtypepk": "0001A810000000002APL",
                "transTypeName": "普通采购",
                "billCode": "",
                "billID": "1001A810000000006JGS",
                "billType": "21",
                "billTypeName": "采购订单",
                "isDirect": true,
                "key": "null211001A810000000006JGS",
                "sourceMap": {},
                "forwardMap": {},
                "sourceRowMap": {},
                "forwardRowMap": {},
                "status": "0",
                "m_isDirty": false
            }
        },
        "sourceRowMap": {},
        "forwardRowMap": {},
        "status": "0",
        "m_isDirty": false
    }, "sourceList": [], "isForward": "-1"
}, {
    "lightBillVO": {
        "transtype": "21-01",
        "transtypepk": "0001A810000000002APL",
        "transTypeName": "普通采购",
        "billCode": "",
        "billID": "1001A810000000006JGS",
        "billType": "21",
        "billTypeName": "采购订单",
        "isDirect": true,
        "key": "null211001A810000000006JGS",
        "sourceMap": {},
        "forwardMap": {},
        "sourceRowMap": {},
        "forwardRowMap": {},
        "status": "0",
        "m_isDirty": false
    },
    "forwardList": ["1001A810000000006JH0", "1001A810000000006JI7", "1001A810000000006JHS"],
    "sourceList": ["1001A810000000006JFM"],
    "isForward": "0"
}, {
    "lightBillVO": {
        "transtype": "23-01",
        "transtypepk": "0001A810000000002APS",
        "transTypeName": "普通到货",
        "billCode": "DH2020072800000001",
        "billID": "1001A810000000006JH0",
        "billType": "23",
        "billTypeName": "到货单",
        "sourceID": "1001A810000000006JGS",
        "billCorp": "0001A810000000002QUK",
        "isDirect": true,
        "key": "DH2020072800000001null1001A810000000006JH0",
        "sourceMap": {
            "null211001A810000000006JGS": {
                "transtype": "21-01",
                "transtypepk": "0001A810000000002APL",
                "transTypeName": "普通采购",
                "billCode": "",
                "billID": "1001A810000000006JGS",
                "billType": "21",
                "billTypeName": "采购订单",
                "isDirect": true,
                "key": "null211001A810000000006JGS",
                "sourceMap": {},
                "forwardMap": {},
                "sourceRowMap": {},
                "forwardRowMap": {},
                "status": "0",
                "m_isDirty": false
            }
        },
        "forwardMap": {},
        "sourceRowMap": {},
        "forwardRowMap": {},
        "status": "0",
        "m_isDirty": false
    }, "forwardList": [], "isForward": "1"
}, {
    "lightBillVO": {
        "transtype": "23-01",
        "transtypepk": "0001A810000000002APS",
        "transTypeName": "普通到货",
        "billCode": "DH2020072800000003",
        "billID": "1001A810000000006JI7",
        "billType": "23",
        "billTypeName": "到货单",
        "sourceID": "1001A810000000006JGS",
        "billCorp": "0001A810000000002QUK",
        "isDirect": true,
        "key": "DH2020072800000003null1001A810000000006JI7",
        "sourceMap": {
            "null211001A810000000006JGS": {
                "transtype": "21-01",
                "transtypepk": "0001A810000000002APL",
                "transTypeName": "普通采购",
                "billCode": "",
                "billID": "1001A810000000006JGS",
                "billType": "21",
                "billTypeName": "采购订单",
                "isDirect": true,
                "key": "null211001A810000000006JGS",
                "sourceMap": {},
                "forwardMap": {},
                "sourceRowMap": {},
                "forwardRowMap": {},
                "status": "0",
                "m_isDirty": false
            }
        },
        "forwardMap": {},
        "sourceRowMap": {},
        "forwardRowMap": {},
        "status": "0",
        "m_isDirty": false
    }, "forwardList": [], "isForward": "1"
}, {
    "lightBillVO": {
        "transtype": "23-01",
        "transtypepk": "0001A810000000002APS",
        "transTypeName": "普通到货",
        "billCode": "DH2020072800000002",
        "billID": "1001A810000000006JHS",
        "billType": "23",
        "billTypeName": "到货单",
        "sourceID": "1001A810000000006JGS",
        "billCorp": "0001A810000000002QUK",
        "isDirect": true,
        "key": "DH2020072800000002null1001A810000000006JHS",
        "sourceMap": {
            "null211001A810000000006JGS": {
                "transtype": "21-01",
                "transtypepk": "0001A810000000002APL",
                "transTypeName": "普通采购",
                "billCode": "",
                "billID": "1001A810000000006JGS",
                "billType": "21",
                "billTypeName": "采购订单",
                "isDirect": true,
                "key": "null211001A810000000006JGS",
                "sourceMap": {},
                "forwardMap": {},
                "sourceRowMap": {},
                "forwardRowMap": {},
                "status": "0",
                "m_isDirty": false
            }
        },
        "forwardMap": {},
        "sourceRowMap": {},
        "forwardRowMap": {},
        "status": "0",
        "m_isDirty": false
    }, "forwardList": [], "isForward": "1"
}];

exports.dagreData = dagreData;
exports.createEdges = createEdges;
exports.getRelated = getRelated;
exports.getSourceTargetNode = getSourceTargetNode;
exports.suitableLabelText = suitableLabelText;
exports.textSize = textSize;