'use strict';

var React = require('react');

var _require = require('yylib-ui'),
    YYClass = _require.YYClass,
    YYPagination = _require.YYPagination,
    YYRadioGroup = _require.YYRadioGroup,
    YYRadioButton = _require.YYRadioButton,
    YYSpin = _require.YYSpin,
    YYCheckbox = _require.YYCheckbox,
    YYMessage = _require.YYMessage,
    YYModal = _require.YYModal,
    YYButton = _require.YYButton;

var _ = require('lodash');
var ajax = require('yylib-utils/ajax');
var PropTypes = YYClass.PropTypes;
var field = YYClass.field;
var uiMeta = {
    name: 'ijz-listview',
    fields: [
    //field参数列表：字段属性名|属性默认值|属性类型|是否必填|属性描述
    field('dataSource', undefined, PropTypes.array, false, '列表的数据'), field('rowKey', 'id', PropTypes.string, false, '列表的主键的字段名称'), field('autoQuery', true, PropTypes.bool, false, '列表加载时是否自动查询数据'), field('queryMethod', 'post', PropTypes.string, false, '查询数据的请求方式'), field('queryUrl', null, PropTypes.string, false, '查询数据的请求URL'), field('queryParam', null, PropTypes.object, false, '查询数据的请求附加的参数'), field('onQueryResult', null, PropTypes.function, false, '查询数据的请求结果的包装回调'), field('onQuerySuccess', null, PropTypes.function, false, '查询数据的请求成功后的回调'), field('onQueryFail', null, PropTypes.function, false, '查询数据的请求失败后的回调'), field('onQueryParam', null, PropTypes.function, false, '查询数据的请求的参数包装回调'), field('deleteUrl', null, PropTypes.string, false, '删除数据的请求URL'), field('deleteParam', null, PropTypes.object, false, '删除数据的请求附加的参数'), field('onDeleteParam', null, PropTypes.function, false, '删除数据的请求的参数包装回调'), field('onDeleteSuccess', null, PropTypes.function, false, '删除数据的请求成功后的回调'), field('onDeleteFail', null, PropTypes.function, false, '删除数据的请求失败后的回调'), field('onDeleteBeforeAction', null, PropTypes.function, false, '删除数据的请求之前的回调，返回true则继续，否则不执行删除'), field('sortGroupList', null, PropTypes.array, false, '列表的排序分组配置'), field('onRowRender', null, PropTypes.function, true, '渲染列表每行的回调'), field('onBatchModeChange', null, PropTypes.function, false, '当批量勾选模式切换时的回调'), field('onExportClick', null, PropTypes.function, false, '导出按钮的点击事件'), field('exportBtnVisible', true, PropTypes.bool, false, '导出按钮的是否显示'), field('deleteBtnVisible', true, PropTypes.bool, false, '删除按钮的是否显示'), field('searchBtnVisible', false, PropTypes.bool, false, '高级查询的是否显示'), field('batchGroupList', null, PropTypes.array, false, '批量按钮配置'), field('onBatchBtnClick', null, PropTypes.func, false, '批量按钮回调，({key})=>{}'), field('cookies', false, PropTypes.bool, false, 'ajax请求是否携带cookie,满足ncc使用需求')]
};
var emptyData = require('./style/empty-data.png');
var IjzBatchBtn = require('./IjzBatchBtn');

//获取默认的cookiekey
function getDefaultCookieKey(props) {
    if (!props || !_.isPlainObject(props)) {
        return '';
    }

    var _ref = props || {},
        cookieKey = _ref.cookieKey,
        uikey = _ref.uikey;

    var _ref2 = props.uiorigin || {},
        pageCode = _ref2.pageCode;

    if (cookieKey) {
        return cookieKey;
    } else {
        //未配置cookieKey时，如果是界面设计器的页面则默认生成pageCode+uiKey作为cookieKey.
        if (pageCode && uikey) {
            return pageCode + uikey;
        }
    }
    return '';
}

var IjzListView = YYClass.create({
    uiMeta: uiMeta,
    getInitialState: function getInitialState() {
        return {
            radioList: this._getSortRadioList(this.props.sortGroupList),
            pageNumber: 1,
            pageSize: YYPagination.getDefaultPageSize(getDefaultCookieKey(this.props)),
            total: 0,
            loading: false,
            batchMode: false,
            viewMode: this.props.viewMode || 'card', //card | table
            dataSource: this.props.dataSource || [], //全部行数据
            selectedRowKeys: [], //被勾选的行键值
            searchCompVisible: false
        };
    },
    //初始化处理排序条件组
    _getSortRadioList: function _getSortRadioList(sortGroupList) {
        var radioList = [];
        _.forEach(sortGroupList || [], function (item, index) {
            var radio = _.assign({
                sort: 'desc',
                isActive: false
            }, item);
            radioList.push(radio);
            if (index === 0) radio.isActive = true;
        });
        return radioList;
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        if ("sortGroupList" in nextProps) {
            if (!_.isEqual(nextProps.sortGroupList, this.props.sortGroupList)) {
                var radioList = this._getSortRadioList(nextProps.sortGroupList);
                this.setState({ radioList: radioList });
            }
        }
        if ('dataSource' in nextProps) {
            this.setState({ dataSource: nextProps.dataSource });
            var isEmptyDataSource = _.isArray(nextProps.dataSource) && nextProps.dataSource.length == 0 || nextProps.dataSource == null || nextProps.dataSource == undefined;
            //空dataSource数组时讲pagination对象重置
            if (isEmptyDataSource) {
                this.setState({
                    pageNumber: 1,
                    total: 0
                });
            }
        }
    },
    componentDidMount: function componentDidMount() {
        var autoQuery = this.props.autoQuery;

        if (autoQuery !== false) {
            this.queryData(); //初始化查询数据
        }
    },
    //获取当前的排序条件
    getSortList: function getSortList() {
        var radioList = this.state.radioList;

        var querySort = [];
        radioList.map(function (item) {
            if (item.isActive) {
                querySort.push(item.field + ':' + item.sort);
                return;
            }
        });
        return querySort;
    },
    /**
     * 执行分页查询表格数据，发送请求给queryUrl
     * options参数列表
     * @param clearSelected 是否清理勾选状态,默认true
     * @param pageNumber 外部手动修改页码
     */
    queryData: function queryData(options) {
        var _ref3 = options || {},
            clearSelected = _ref3.clearSelected,
            pageNumber = _ref3.pageNumber;

        var _props = this.props,
            queryMethod = _props.queryMethod,
            queryUrl = _props.queryUrl,
            queryParam = _props.queryParam,
            onQueryResult = _props.onQueryResult,
            onQuerySuccess = _props.onQuerySuccess,
            onQueryFail = _props.onQueryFail,
            onQueryParam = _props.onQueryParam,
            cookies = _props.cookies;

        var that = this;
        var _params = {};
        //查询接口URL
        var _url = queryUrl;
        //分页查询参数
        var pageSize = this.state.pageSize;

        pageNumber = pageNumber ? pageNumber : this.state.pageNumber;
        //需要排序的字段
        var querySort = this.getSortList();
        //常规分页参数
        var defaultParams = {
            pageNumber: pageNumber - 1,
            pageSize: pageSize,
            resultType: 'JSON'
        };
        //通用排序参数
        if (_.isArray(querySort) && querySort.length > 0) {
            defaultParams.orderItems = querySort;
            //兼容旧模式
            defaultParams.sort = querySort.map(function (sort) {
                var sortArr = sort.split(':');
                return { field: sortArr[0], sort: sortArr[1] };
            });
        }
        _params = _.assign(_params, defaultParams, _.isFunction(queryParam) ? queryParam() : queryParam);
        //查询参数的回调
        if (_.isFunction(onQueryParam)) {
            _params = onQueryParam(_params);
        }
        if (!_url) return;
        //默认清理勾选状态
        if (clearSelected !== false) {
            this.clearSelectRowKeys();
        }
        //开启遮罩|取消批量勾选模式
        this.setState({ loading: true, batchMode: false });
        //执行查询
        var callAjax;
        if (queryMethod == 'get') {
            callAjax = !!cookies ? ajax.getJSONWithCookies : ajax.getJSON;
        } else {
            callAjax = !!cookies ? ajax.postJSONWithCookies : ajax.postJSON;
        }
        callAjax.call(ajax, _url, _params, function (result) {
            //包装分页查询的结果回调
            if (_.isFunction(onQueryResult)) {
                result = onQueryResult(result);
            }
            var backMsg = result && result.backMsg ? result.backMsg : null;
            //java端返回的格式为JsonBackData+Page格式
            if (result && result.success) {
                var dataSource = result.backData;
                if (dataSource) {
                    //本页数据
                    that.state.dataSource = dataSource.content;
                    //分页数据
                    that.state.pageNumber = dataSource.number + 1;
                    that.state.pageSize = dataSource.size;
                    that.state.total = dataSource.totalElements;
                } else {
                    that.state.dataSource = [];
                    that.setState();
                }
            } else {
                //显示后端自定义错误提示
                if (backMsg) YYMessage.error(backMsg);
            }
            //成功回调外部事件
            if (_.isFunction(onQuerySuccess)) {
                onQuerySuccess(result);
            }
            //关闭遮罩
            that.state.loading = false;
            that.setState();
        }, function () {
            YYMessage.error('查询数据失败');
            //失败回调外部事件
            if (_.isFunction(onQueryFail)) {
                onQueryFail();
            }
            //关闭遮罩
            that.state.loading = false;
            that.setState();
        });
    },
    onBatchModeChange: function onBatchModeChange(batchMode) {
        this.setState({ batchMode: batchMode });
    },
    onViewChange: function onViewChange() {
        var viewMode = this.state.viewMode;

        if (viewMode === 'card') {
            this.setState({ viewMode: 'table' });
        } else if (viewMode === 'table') {
            this.setState({ viewMode: 'card' });
        }
    },
    onSearchClick: function onSearchClick() {
        this.setState({ searchCompVisible: !this.state.searchCompVisible });
    },
    /**
     * 获取当前勾选的行记录键值
     * @returns array[string]
     */
    getSelectedRowKeys: function getSelectedRowKeys() {
        return this.state.selectedRowKeys;
    },
    /**
     * 清除全部勾选状态
     */
    clearSelectRowKeys: function clearSelectRowKeys() {
        this.setState({ selectedRowKeys: [] });
    },
    /**
     * 获取当前勾选的行数据
     */
    getSelectedRowData: function getSelectedRowData() {
        var rowKey = this.props.rowKey;

        var selectedRowKeys = this.getSelectedRowKeys();
        var rowData = this.getDataSource();
        //匹配出当前勾选行数据
        var selectedRowData = _.filter(rowData, function (row) {
            if (_.includes(selectedRowKeys, row[rowKey])) {
                return row;
            }
        });
        return selectedRowData;
    },
    /**
     * 获取表格数据源
     */
    getDataSource: function getDataSource() {
        return this.state.dataSource;
    },
    //获取分页信息
    getPagination: function getPagination() {
        var _state = this.state,
            pageNumber = _state.pageNumber,
            pageSize = _state.pageSize,
            total = _state.total;

        return { pageNumber: pageNumber, pageSize: pageSize, total: total };
    },
    getReactElement: function getReactElement(type) {
        var children = this.props.children;

        var content = React.createElement('div', null);
        React.Children.map(children, function (item) {
            if (React.isValidElement(item) && item.type.displayName === type) {
                content = item;
            }
        });
        return content;
    },
    //排序按钮组切换事件
    _radioChange: function _radioChange(item, index) {
        var radioList = this.state.radioList;

        if (item.isActive) {
            item.sort = item.sort === 'asc' ? 'desc' : 'asc';
        } else {
            radioList.map(function (radio, index) {
                radio.isActive = false;
            });
            item.isActive = true;
        }
        radioList[index] = item;
        this.setState({ radioList: radioList, pageNumber: 1 }, this.queryData);
    },
    onShowSizeChange: function onShowSizeChange(current, pageSize) {
        var _this = this;

        this.setState({
            pageNumber: current,
            pageSize: pageSize
        }, function () {
            _this.queryData();
        });
    },
    //页码改变时的回调，参数为改变后的页码
    onSizeChange: function onSizeChange(pageNumber) {
        var _this2 = this;

        this.setState({
            pageNumber: pageNumber
        }, function () {
            _this2.queryData();
        });
    },
    //用于显示总共有多少条数据
    showTotal: function showTotal(total) {
        return '\u5171 ' + total + ' \u6761';
    },
    /**
     * 将勾选行的数据发送请求给deleteUrl
     */
    deleteData: function deleteData(options) {
        var _ref4 = options || {},
            onConfirm = _ref4.onConfirm,
            id = _ref4.id;

        var _props2 = this.props,
            deleteUrl = _props2.deleteUrl,
            rowKey = _props2.rowKey,
            deleteParam = _props2.deleteParam,
            onDeleteParam = _props2.onDeleteParam,
            onDeleteSuccess = _props2.onDeleteSuccess,
            onDeleteFail = _props2.onDeleteFail,
            onDeleteBeforeAction = _props2.onDeleteBeforeAction;

        var that = this;
        //获取行勾选的数据键值
        var ids = id ? [id] : this.getSelectedRowKeys();
        if (ids && ids.length) {
            var confirmOk = function confirmOk() {
                var _params = deleteBean;
                _params = _.assign(_params, _.isFunction(deleteParam) ? deleteParam() : deleteParam);
                //删除参数重写的回调
                if (_.isFunction(onDeleteParam)) {
                    _params = onDeleteParam(_params);
                }
                //删除接口URL
                var _url = deleteUrl;
                if (!_url) return;
                //执行删除
                ajax.postJSON(_url, _params, function (result) {
                    //显示后端自定义错误提示
                    var backMsg = result && result.backMsg ? result.backMsg : "数据删除失败";
                    if (result && result.success) {
                        //删除成功
                        //重新刷新列表
                        that.queryData();
                    } else {
                        YYMessage.error(backMsg);
                    }
                    //成功回调外部事件
                    if (_.isFunction(onDeleteSuccess)) {
                        onDeleteSuccess(result);
                    }
                }, function () {
                    //失败回调外部事件
                    if (_.isFunction(onDeleteFail)) {
                        onDeleteFail();
                    }
                    YYMessage.error('数据删除失败');
                });
            };

            var deleteBean = [];
            //审批流限制-只能删除自由态的单据
            for (var i = 0; i < ids.length; i++) {
                var data = _.find(that.state.dataSource, [rowKey, ids[i]]);
                var result = true;
                if (onDeleteBeforeAction && _.isFunction(onDeleteBeforeAction)) {
                    result = onDeleteBeforeAction(ids, data);
                }
                if (!result) {
                    return;
                }
                if (data.billState !== 0) {
                    YYMessage.info('存在非自由态的单据，不能删除！', 3);
                    return;
                }
                if (data.ts) {
                    //删除的内容id+ts
                    var bean = {};
                    bean.ts = data.ts;
                    bean[rowKey] = data[rowKey];
                    deleteBean.push(bean);
                } else {
                    //删除的ids
                    deleteBean.push(data[rowKey]);
                }
            }

            ;
            var confirmContent = _.assign({
                title: '删除确认',
                content: '您确定要删除这' + ids.length + '行数据?',
                onOk: confirmOk
            }, typeof onConfirm == 'function' ? onConfirm(ids) : onConfirm);
            YYModal.confirm(confirmContent);
        } else {
            YYMessage.error('请勾选需要删除的数据');
        }
    },
    //全选按钮
    onCheckAllChange: function onCheckAllChange(e) {
        var rowKey = this.props.rowKey;

        var checked = e.target.checked;
        var selectedRowKeys = [];
        if (checked) {
            //全勾选
            _.forEach(this.state.dataSource || [], function (row, index) {
                selectedRowKeys.push(row[rowKey]);
            });
        } else {
            //全取消
            selectedRowKeys = [];
        }
        this.setState({ selectedRowKeys: selectedRowKeys });
    },
    //行勾选
    onRowCheckboxChange: function onRowCheckboxChange(rowId, event) {
        var checked = event.target.checked;
        var selectedRowKeys = this.state.selectedRowKeys;

        if (checked) {
            if (_.indexOf(selectedRowKeys, rowId) === -1) {
                selectedRowKeys.push(rowId);
            }
        } else {
            selectedRowKeys = _.remove(selectedRowKeys, function (n) {
                return n !== rowId;
            });
        }
        this.setState({ selectedRowKeys: selectedRowKeys });
    },
    //渲染排序组件视图
    _renderSortViews: function _renderSortViews() {
        var _this3 = this;

        var radioList = this.state.radioList;

        return React.createElement(
            YYRadioGroup,
            { className: 'ijz-listview-order', defaultValue: radioList[0] ? radioList[0].field : null },
            radioList.map(function (item, index) {
                return React.createElement(
                    YYRadioButton,
                    { onClick: _this3._radioChange.bind(null, item, index),
                        value: item.field },
                    item.name,
                    item.sort === 'desc' ? React.createElement('i', { className: 'iconfont icon-paixu-jiang',
                        style: { marginLeft: 5, fontSize: 10 } }) : React.createElement('i', { className: 'iconfont icon-paixu-sheng',
                        style: { marginLeft: 5, fontSize: 10 } })
                );
            })
        );
    },
    //渲染空内容数据视图
    _renderEmptyView: function _renderEmptyView() {
        var _props3 = this.props,
            uiRunMode = _props3.uiRunMode,
            children = _props3.children;
        var viewMode = this.state.viewMode;

        if (uiRunMode === 'design' && _.isArray(children)) {
            return viewMode === 'card' ? React.cloneElement(this.getReactElement('IjzListItem'), { uiRunMode: uiRunMode }) : React.cloneElement(this.getReactElement('TableWidget'), { uiRunMode: uiRunMode });
        } else {
            return React.createElement(
                'div',
                { className: 'nodata', style: { padding: '100px 0', textAlign: 'center', backgroundColor: '#f6f6f6' } },
                React.createElement('img', { src: emptyData, style: { width: 162 } }),
                React.createElement(
                    'div',
                    { style: { color: '#999' } },
                    '~\u6682\u65E0\u6570\u636E~'
                )
            );
        }
    },
    //渲染列表内容数据行
    _renderDataList: function _renderDataList() {
        var _this4 = this;

        var _props4 = this.props,
            onRowRender = _props4.onRowRender,
            rowKey = _props4.rowKey,
            children = _props4.children,
            viewModeBtnVisible = _props4.viewModeBtnVisible;
        var _state2 = this.state,
            dataSource = _state2.dataSource,
            batchMode = _state2.batchMode,
            selectedRowKeys = _state2.selectedRowKeys,
            viewMode = _state2.viewMode;
        //渲染列表项数据内容

        var itemViews = [];
        _.forEach(dataSource || [], function (item, index) {
            var rowId = item[rowKey] || index;
            var itemContentView = null;
            if (typeof onRowRender == 'function') {
                itemContentView = onRowRender({ rowData: item, batchMode: batchMode });
            } else if (_.isArray(children)) {
                itemContentView = viewMode === 'card' ? React.cloneElement(_this4.getReactElement('IjzListItem'), { dataItem: item, batchMode: batchMode }) : null;
            } else {
                itemContentView = React.createElement(
                    'p',
                    null,
                    "请实现onRowRender方法，渲染第" + (index + 1) + "行数据视图"
                );
            }
            var itemRowView = React.createElement(
                'div',
                { className: 'ijz-listview-table-row', key: "row_" + rowId },
                batchMode == true ? React.createElement(YYCheckbox, { className: 'ijz-listview-table-row-checkbox',
                    onChange: _this4.onRowCheckboxChange.bind(_this4, rowId),
                    checked: _.indexOf(selectedRowKeys, rowId) > -1 }) : null,
                itemContentView ? itemContentView : null
            );
            itemViews.push(itemRowView);
        });
        return viewMode === 'table' ? React.cloneElement(this.getReactElement('TableWidget'), {
            dataSource: dataSource,
            rowSelection: { selectedRowKeys: selectedRowKeys, onChange: function onChange(Keys) {
                    return _this4.state.selectedRowKeys = Keys;
                } }
        }) : itemViews;
    },
    render: function render() {
        var _state3 = this.state,
            dataSource = _state3.dataSource,
            batchMode = _state3.batchMode,
            pageSize = _state3.pageSize,
            pageNumber = _state3.pageNumber,
            total = _state3.total,
            viewMode = _state3.viewMode,
            searchCompVisible = _state3.searchCompVisible,
            selectedRowKeys = _state3.selectedRowKeys;
        //渲染列表项数据内容

        var itemViews = this._renderDataList();
        return React.createElement(
            'div',
            { className: 'ijz-listview-layout' },
            React.createElement(
                YYSpin,
                { size: 'large', spinning: this.state.loading },
                React.createElement(
                    'div',
                    { className: 'ijz-listview-header' },
                    React.createElement(IjzBatchBtn, {
                        checked: dataSource.length === selectedRowKeys.length,
                        batchMode: batchMode,
                        viewMode: viewMode,
                        onCheckAllChange: this.onCheckAllChange,
                        onBatchModeChange: this.onBatchModeChange,
                        onDeleteClick: this.deleteData,
                        onExportClick: this.props.onExportClick,
                        exportBtnVisible: this.props.exportBtnVisible,
                        deleteBtnVisible: this.props.deleteBtnVisible,
                        searchBtnVisible: this.props.searchBtnVisible,
                        batchGroupList: this.props.batchGroupList,
                        onBatchBtnClick: this.props.onBatchBtnClick,
                        onSearchClick: this.onSearchClick
                    }),
                    this._renderSortViews(),
                    React.createElement(
                        'div',
                        { style: { fontSize: 12 } },
                        '\u5171 ',
                        React.createElement(
                            'span',
                            { style: { color: '#F83F3F' } },
                            total
                        ),
                        ' \u4EFD\u76F8\u5173\u8BB0\u5F55'
                    ),
                    this.props.viewModeBtnVisible ? React.createElement(YYButton, { className: "view-mode-" + viewMode, onClick: this.onViewChange }) : null
                ),
                React.cloneElement(this.getReactElement('IjzSearch'), { visible: searchCompVisible }),
                React.cloneElement(this.getReactElement('NccAdvanceSearch'), { visible: true }),
                dataSource && dataSource.length > 0 ? React.createElement(
                    'div',
                    null,
                    React.createElement(
                        'div',
                        { className: 'ijz-listview-center' },
                        itemViews
                    ),
                    React.createElement(
                        'div',
                        { className: 'ijz-listview-footer' },
                        React.createElement(YYPagination, {
                            cookieKey: getDefaultCookieKey(this.props),
                            defaultCurrent: 1,
                            current: pageNumber,
                            pageSize: pageSize,
                            showTotal: this.showTotal,
                            total: total,
                            showQuickJumper: true,
                            showSizeChanger: true,
                            onShowSizeChange: this.onShowSizeChange,
                            onChange: this.onSizeChange
                        })
                    )
                ) : this._renderEmptyView()
            )
        );
    }
});
module.exports = IjzListView;